import 'package:flutter/material.dart';

class SectionGroupWidget<T> {
  final T value;
  final Widget child;
  SectionGroupWidget({required this.value, required this.child});
}

class SectionGroup<T> extends StatelessWidget {
  final Widget? header;
  final Widget? footer;
  final List<SectionGroupWidget> children;
  final CrossAxisAlignment crossAxisAlignment;
  late final EdgeInsetsGeometry margin;
  late final EdgeInsetsGeometry contentPadding;
  final double? elevation;
  final Color? dividerColor;

  final T? groupValue;
  final ValueChanged<T?>? onChanged;

  // ignore: non_constant_identifier_names
  static double DEFAULT_CONTENT_PADDING = 10.0;

  SectionGroup({
    Key? key,
    this.header,
    required this.groupValue,
    required this.onChanged,
    required this.children,
    this.footer,
    this.crossAxisAlignment = CrossAxisAlignment.start,
    EdgeInsets? margin,
    EdgeInsets? headerPadding,
    this.elevation,
    this.dividerColor,
  }) : super(key: key) {
    this.margin =
        margin ?? EdgeInsets.fromLTRB(DEFAULT_CONTENT_PADDING, DEFAULT_CONTENT_PADDING, DEFAULT_CONTENT_PADDING, 0);
    this.contentPadding = headerPadding ?? EdgeInsets.fromLTRB(0, 0, 0, DEFAULT_CONTENT_PADDING);
  }

  @override
  Widget build(BuildContext context) {
    return Padding(
      padding: margin,
      child: Column(
        crossAxisAlignment: crossAxisAlignment,
        children: [
          if (header != null)
            // Padding(
            //   padding: contentPadding,
            //   child: header,
            // ),
            header!,
          Card(
            margin: EdgeInsets.zero,
            elevation: elevation,
            child: Column(
              crossAxisAlignment: crossAxisAlignment,
              children: [
                ...children.map((e) {
                  return RadioListTile<T>(
                    value: e.value,
                    groupValue: groupValue,
                    onChanged: onChanged,
                    title: e.child,
                    controlAffinity: ListTileControlAffinity.trailing,
                  );
                })
              ],
            ),
            // child: ListView.separated(
            //   itemBuilder: (context, index) {
            //     return children[index];
            //   },
            //   separatorBuilder: (context, index) {
            //     return Divider(height: 1);
            //   },
            //   itemCount: 3,
            // ),
          ),
          if (footer != null)
            Padding(
              padding: contentPadding,
              child: footer,
            ),
        ],
      ),
    );
  }
}
